% "Using geographically weighted choice models to account for the spatial heterogeneity of preferences"
% Wiktor Budziski, Danny Campbell, Mikoaj Czajkowski, Urka Demar and Nick Hanley

% Online codes
% 2017-09-09 version

% Table 3 Correlation coefficients of model-specific predicted probabilities of the observed choices (Ben-Akiva-Lermans pseudo-R2)

% Input
% File GWMNL_results.mat, which can be calculated using GWMNLcalc.m code
% File MXL_results_new.mat, which can be calculated using MXLcalc.m code
% File polforex_2.9.mat, which contains data for attributes used in DCE

%% Calculations

% clearing working space
clear
clc
rng(179424673);

% Reading attributes 
EstimOpt.fnameinput = ('polforex_2.9.mat'); 
DATA = load(EstimOpt.fnameinput);
Y = DATA.ATT.ANS == 1;
X = [DATA.ATT.ALT == 1, DATA.ATT.N == 1, DATA.ATT.N == 2, DATA.ATT.S == 1, DATA.ATT.S == 2, DATA.ATT.I == 1, DATA.ATT.I == 2, -DATA.ATT.K/4];
Crds = DATA.Crds2;


%% For GWR
tic
load('GWMNL_results.mat');
B = Results.GWMNL1.bhat;
B(1:7,:) = B(1:7,:).*B(8*ones(1,7),:); % transforming into WTP space

CrdsUni = unique(Crds, 'rows'); % Unique coordinates
NoReg = size(CrdsUni,1); % No of locations
X = reshape(X, 4*26, 1001, 8);
Y = reshape(Y, 4*26, 1001);
P_gwr = NaN(26,1001); % Probability of choice
Pred_gwr = NaN(1001,1);
P_loc = NaN(NoReg, 1);

for i = 1:NoReg
    FindIndx = find(Crds(:,1) == CrdsUni(i,1) & Crds(:,2) == CrdsUni(i,2));
    BX = B(:,FindIndx(1));
    XX = reshape(X(:, FindIndx,:),4*26*length(FindIndx),8);
    Fit = reshape(exp(XX*BX), 4, 26, length(FindIndx));
    sFit = sum(Fit,1);
    PP = Fit./sFit(ones(1,4),:,:); % MNL probability for ith location
    YY = reshape(Y(:, FindIndx),4*26*length(FindIndx),1);
    PP = reshape(PP(YY == 1), 26, length(FindIndx));
    P_gwr(:, FindIndx) = PP;
    P_loc(i) = prod(PP(:)).^(1./(26*length(FindIndx)));
    Pred_gwr(FindIndx) = P_loc(i);
end
%P = reshape(P, 26*1001,1);

toc

%% For Location - specific MXL
tic
% Model parameters
load('MXL_results_new.mat');
bhat = Results.MXL2.bhat;
%bhat = [-3.24606790448899;11.7961127398206;16.8629773941378;17.4477287232644;25.2328654604006;8.26411316068301;12.1144694239853;-2.24723751571725;43.2614633903624;-3.35703464514994;-6.04513918870589;-2.55009350733115;-6.43224222633824;-0.458989551225704;-0.674435095217947;-0.0782838703189936;6.80299463623996;10.5054933483353;3.42178290998973;6.15158373082512;0.156042247782263;1.37536919095658;-0.332724311075367;0.864436639017184;5.05736463849861;8.70792643335967;0.804407555459866;1.51862696613339;-0.0853115941855643;4.99068388590264;6.03117616614865;3.97863677118963;5.68299967048991;-0.439018000889176;-2.70503234413649;1.14663858226285;-0.356723420248356;0.00574817860340791;-1.45835819472106;-1.76326796486317;-0.244278846054378;-1.02637168866727;-0.322784737033579;-0.112389476475518];
CrdsUni = unique(Crds, 'rows'); 
NoReg = size(CrdsUni,1);

% Generating random numbers
hm1 = sobolset(8,'Skip',1,'Leap',0); 
hm1 = scramble(hm1,'MatousekAffineOwen');
err_mtx = net(hm1,NoReg*1000); % this takes every point:
clear hm1;
err_mtx = icdf('Normal',err_mtx,0,1);
err_mtx = err_mtx';
ba = bhat(1:8);
VC = tril(ones(8));
VC(VC == 1) = bhat(9:end);
bmxl = ba(:, ones(1, NoReg*1000)) + VC*err_mtx;
bmxl(8,:) = exp(bmxl(8,:));
bmxl(1:7,:) = bmxl(1:7,:).*bmxl(8*ones(1,7),:); % Because model estimates in WTP-space
bmxl = reshape(bmxl,8,1000,NoReg); 

P2 = NaN(NoReg,1);
Pred_mxl = NaN(1001,1);
for i = 1:NoReg
    FindIndx = find(Crds(:,1) == CrdsUni(i,1) & Crds(:,2) == CrdsUni(i,2));
    BX = bmxl(:,:,i);
    XX = reshape(X(:, FindIndx,:),4*26*length(FindIndx),8);
    U = reshape(XX*BX,4,26*length(FindIndx),1000);
    U_max = max(U);
    U = exp(U - U_max(ones(4,1),:,:)); % rescale utility to avoid exploding
    U_sum = reshape(sum(U,1),26*length(FindIndx),1000);
    YY = reshape(Y(:, FindIndx),4*26*length(FindIndx),1);
    U_selected = reshape(U(YY(:,ones(1000,1))==1),26*length(FindIndx),1000);
    PP = mean(prod(U_selected ./ U_sum,1));
    P2(i) = PP.^(1./(26*length(FindIndx)));
    Pred_mxl(FindIndx) = P2(i);
end
toc


%% MNL 
% This is for preference-space
tic
bhat = [2.00301091436139;0.797603252552557;1.17352894807849;1.43430609766328;1.91878856965426;0.652887231950428;1.05192928366473;0.0537803804823240];
X = reshape(X, 4*26*1001, 8);
Y = reshape(Y, 4*26*1001,1);
Fit = reshape(exp(X*bhat), 4, 26*1001);
sFit = sum(Fit,1);
P3 = Fit./sFit(ones(4,1),:);
P3 = P3(Y == 1);
P3 = reshape(P3, 26,1001);

P_loc3 = NaN(NoReg,1);
Pred_mnl = NaN(1001,1);
for i = 1:NoReg
    FindIndx = find(Crds(:,1) == CrdsUni(i,1) & Crds(:,2) == CrdsUni(i,2));
    PP = P3(:, FindIndx);
    P_loc3(i) = prod(PP(:)).^(1./(26*length(FindIndx)));
    Pred_mnl(FindIndx) = P_loc3(i);
end
toc

C = corr([P_loc,P_loc3, P2]);
Out = num2cell(C);
xlswrite('GWR_Tab3.xls', Out);
 
 ProbComp = [Pred_gwr, Pred_mxl, Pred_mnl]; % This is used by R codes to plot Figure 2