% "Using geographically weighted choice models to account for the spatial heterogeneity of preferences"
% Wiktor Budziski, Danny Campbell, Mikoaj Czajkowski, Urka Demar and Nick Hanley

% Online codes
% 2017-09-09 version

% Table 4 Results of regressions in which WTP estimates from GWMNL are explained by GIS variables

% This code uses Tools package from https://github.com/czaj/Tools

% Input
% File GWMNL_results.mat, which can be calculated using GWMNLcalc.m code
% File Spatial_20160223.mat, which contains GIS and socio-demographic
% variables
% File polforex_2.8.mat, which contains Latitude and Longitude coordinates

%% Calculations

% clearing working space
tic
clear
clc

% reading datasets

sociodemo = 0; % = 1 if include socio-demographic; % = 2 if only socio demographics
% Results from GWMNL
load('GWMNL_results.mat');
% File with GIS variables
load('Spatial_20160223.mat')

% changing order to match order in the paper
tmp = [14, 20, 30, 33:36];
DVar = Spatial(:,tmp);
DVar = DVar(:, [6, 5, 7, 3, 2, 1,4]);

% Scalling of the variables
DVar(:, [1:4, 6:7]) = DVar(:, [1:4, 6:7])/100;
DVar(:,5) = DVar(:,5)/1000; % EuclAvg

% Estimates from GWMNL
B_wtp = Results.GWMNL1.bhat(1:7,:);
X = [ones(1001,1), DVar];

% Transforming dataset from 1001 observations (multiple observations for
% given location) to 253 format (one observation per location)

XX = X;
% Reading file with model coordinates 

load('polforex_2.8.mat');
Crds = [GIS1.Longitude(1:4*26:end), GIS1.Latitude(1:4*26:end)];
B_wtpX = B_wtp;
CrdsUni = unique(Crds, 'rows'); 
NoReg = size(CrdsUni,1);
B_wtp = zeros(7,NoReg);
X = zeros(NoReg,8);

for i = 1:NoReg
    FindIndx = find(Crds(:,1) == CrdsUni(i,1) & Crds(:,2) == CrdsUni(i,2));
    B_wtp(:,i) = B_wtpX(:,FindIndx(1));
    X(i,:) = XX(FindIndx(1),:);
end

% Analysis of collinearity 
% To use this you need Collinearity diagnostics package by Brian Lau

% info = colldiag(X(:,2:end)); % calculate collinearity diagnostics on design matrix x [nSamples x nVariables]
% disp(info.str) % print info to stdout
% colldiag_tableplot(info); % collinearity tableplot


if sociodemo > 0
    load('DecompVars.mat') % Reading additional socio-demographic variables
    EduHigh =Edu >9;
    logP16 =log(P16);
    logP18 =log(P18+1);
    SD = [Age(2:end), EduHigh(2:end), M21(2:end), logP16(2:end), logP18(2:end)];
    SD_t = zeros(NoReg, 5);
    for i = 1:NoReg
        FindIndx = find(Crds(:,1) == CrdsUni(i,1) & Crds(:,2) == CrdsUni(i,2));
        SD_t(i,:) = mean(SD(FindIndx,:),1);
    end
end

X = X(:,2:end);

if sociodemo == 0
    T = zeros(8*2,7);
    P = zeros(8*2,7);
    R2 = zeros(7,1);

elseif sociodemo == 1
    T = zeros((8+5)*2,7);
    P = zeros((8+5)*2,7);
    R2 = zeros(7,1);
    TestSocio = zeros(7,1);
    X = [X, SD_t];
%     info = colldiag(X(:,2:end)); % calculate collinearity diagnostics on design matrix x [nSamples x nVariables]
%     disp(info.str) % print info to stdout
%     colldiag_tableplot(info); % collinearity tableplot
elseif sociodemo == 2
    T = zeros((6)*2,7);
    P = zeros((6)*2,7);
    R2 = zeros(7,1);
    X = SD_t;
end

for j = 1:7
    res=fitlm(X, B_wtp(j,:)');
    if sociodemo == 1
        H = [zeros(1,8), 1, zeros(1,4); zeros(1,9), 1, zeros(1,3); zeros(1,10), 1, zeros(1,2); ...
            zeros(1,11), 1, zeros(1,1); zeros(1,12), 1];
        TestSocio(j) = coefTest(res,H);
    end
    R2(j) = res.Rsquared.Ordinary;
    stdx = sqrt(diag(res.CoefficientCovariance));
    COV1 = res.CoefficientCovariance;
    T(1:2:end,j) =table2array(res.Coefficients(:,1));
    T(2:2:end,j) =stdx;
    pval = pv(T(1:2:end,j), stdx);
    P(1:2:end, j) =pval;

end

%% Creating table for export (only for the case considered in the final paper)

if sociodemo == 0
    Out = cell(size(T,1)+ 3,7);
    for j = 1:7
        for i = 1:size(T,1)
            
            if rem(i,2) == 1
                pval = P(i,j);
                if pval < 0.01
                     Out(i,j) = {[num2str(T(i,j), '%8.2f'), '***']}; 
                elseif pval <= 0.05 && pval > 0.01
                    Out(i,j) = {[num2str(T(i,j), '%8.2f'), '** ']}; 
                elseif pval <= 0.1 && pval > 0.05
                    Out(i,j) = {[num2str(T(i,j), '%8.2f'), '*  ']}; 
                elseif pval > 0.1
                    Out(i,j) = {[num2str(T(i,j), '%8.2f'), '   ']}; 
                end
                
            else
                Out(i,j) = {['''(', num2str(T(i,j), '%8.2f'), ')']}; 
            end
        end
        Out(size(T,1)+1,j) = {num2str(R2(j), '%8.2f')}; 
        Out(size(T,1)+2,j) = {'253'};
        Out(size(T,1)+3,j) = {'8'};
    end
    xlswrite('GWR_Tab4.xls', Out);
end

toc